#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include <util/delay.h>

#define LCD_ADDR volatile uint8_t 
#define NULL (void *)0

/* #include "display.h" */
#include "znaky.h"

inline void Display_Init(void){
	LCDCRB = 0xB7;
	LCDFRR = 0x10;
	LCDCCR = 0x0F;
	LCDCRA = 0x80;
}

/* Jednotlive segmenty jsou ve sloupcich LCDDRx,
   na danou adresu se zapisuje byte jehoz spodni
   slabika obsahuje jeden znak a horni slabika
   druhy znak.
 */

LCD_ADDR *Display_GetColumnBase(const char Column){
	/* vybrani adresy daneho segmentu */
	switch(Column){
		case 0:
			return &LCDDR0;
		case 1:
			return &LCDDR0;
		case 2:
			return &LCDDR1;
		case 3:
			return &LCDDR1;
		case 4:	
			return &LCDDR2;
		case 5:
			return &LCDDR2;
	}
}

void Display_Char(const char Letter, const char Column){
	LCD_ADDR *base_addr;
	unsigned char *letter_definition;
	unsigned char old_char[4];
	
	letter_definition = znaky[Letter - 42]; 

	base_addr = Display_GetColumnBase(Column);
	
	if(Column & 1){ /* lichy sloupec */
		/* nutno posunout ukladanou hodnotu na horni pulslabiku */
		*(base_addr + 0)  = letter_definition[0]<<4 | ( *(base_addr + 0) & 0x0f );
		*(base_addr + 5)  = letter_definition[1]<<4 | ( *(base_addr + 5) & 0x0f );
		*(base_addr + 10) = letter_definition[2]<<4 | ( *(base_addr + 10) & 0x0f );
		*(base_addr + 15) =	letter_definition[3]<<4 | ( *(base_addr + 15) & 0x0f );
	}
	else { /* sudy sloupec */
		*(base_addr + 0)  = letter_definition[0] | ( *(base_addr + 0) & 0xf0 );
		*(base_addr + 5)  = letter_definition[1] | ( *(base_addr + 5) & 0xf0 );
		*(base_addr + 10) = letter_definition[2] | ( *(base_addr + 10) & 0xf0 );
		*(base_addr + 15) =	letter_definition[3] | ( *(base_addr + 15) & 0xf0 );
		
	}
	return;
}

void	Display_Clear(){
	int i;
	LCD_ADDR *base_addr;
	
	for(i = 0; i <=5; i++){
		base_addr = Display_GetColumnBase(i);
		*(base_addr + 0)  = 0x00;
		*(base_addr + 5)  = 0x00;
		*(base_addr + 10) = 0x00;
		*(base_addr + 15) = 0x00;
	}
	return;
}

int
main (void)
{
	unsigned char *text = "*_HELL_ON_FEL_*";
	unsigned char l_text = 15; 
	unsigned char i = 0, j = 0;
	Display_Init();

	for(;;){
		Display_Char(text[(j) % l_text], i );
		Display_Char(text[(j+1) % l_text], (i+1) );
		Display_Char(text[(j+2) % l_text], (i+2) );
		Display_Char(text[(j+3) % l_text], (i+3) );
		Display_Char(text[(j+4) % l_text], (i+4) );
		j++;	
		
/*		i--; */	
		_delay_loop_2(0xFFFF);	
		_delay_loop_2(0xFFFF);	
		_delay_loop_2(0xFFFF);	
	}

    for (;;)		
        sleep_mode();

    return (0);
}
